/* Seeburg wallbox driver */
/* Copyright (c) 2007, Scott Swazey.  All rights reserved */
/* Version 1.0 Aug 06, 2007  Initial version		*/

#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <asm/semaphore.h>

extern const char *notify_sound (void);	// from notify.c
extern void hijack_serial_rx_insert (const char *buf, int size, int port); // drivers/char/serial_sa1100.c

#define SEEBURG_DEV 51
#define MAX_SAMPLES 256		//hack -- must match size in <asm/arch/time.h>

unsigned long pulse_samples[MAX_SAMPLES];
long volatile samples_collected = 0;
long volatile sampler_state = 0;
struct semaphore seeburg_sem = MUTEX_LOCKED;

int __init seeburg_init (void);
void seeburg_cleanup (void);

static int seeburg_open (struct inode *inode, struct file *filep);
static ssize_t seeburg_read (struct file *filep, char *buff, size_t count, loff_t * offp);
static ssize_t seeburg_write (struct file *filep, const char *buff, size_t count, loff_t * offp);
static int seeburg_release (struct inode *inode, struct file *filep);

struct file_operations seeburg_fops = {
  open:seeburg_open,
  read:seeburg_read,
  write:seeburg_write,
  release:seeburg_release,
};


int __init seeburg_init (void)
{
  printk ("Seeburg Wallbox driver Version 1.0 by Scott Swazey\n");
  if (register_chrdev (SEEBURG_DEV, "wallbox", &seeburg_fops))
    {
      printk ("Failed to register Seeburg wallbox device\n");
    }
  return 0;
}

void seeburg_cleanup (void)
{
  sampler_state = 0;
  seeburg_sem = MUTEX_LOCKED;
  sampler_state = 0;
  unregister_chrdev (SEEBURG_DEV, "wallbox");
  return;
}

int seeburg_open (struct inode *inode, struct file *filep)
{
  if (sampler_state != 2)
    {
      seeburg_sem = MUTEX_LOCKED;
      sampler_state = 1;
    }
  return 0;
}

ssize_t seeburg_read (struct file * filep, char *buff, size_t count, loff_t * offp)
{
  down_interruptible (&seeburg_sem);	// wait for data
  if (sampler_state == 3)
    {
      if (copy_to_user (buff, pulse_samples, sizeof (pulse_samples)) != 0)
	{
	  printk ("/dev/wallbox userspace copy failed.\n");
	  return 0;
	}
      sampler_state = 4;
      return sizeof (pulse_samples);
    }
  return 0;
}

ssize_t seeburg_write (struct file * filep, const char *buff, size_t count, loff_t * offp)
{
  int len;
  char cmd_buf[20];
  unsigned long fid;
  const char *p_sound;

  if (count != 4)
    return 0;	// wrong size, one fid only

  fid = *((unsigned long *) buff);	//get fid

  p_sound = notify_sound();	//get status of player

  if (p_sound[0] != '1')
    // player is paused, start new playlist
    len = sprintf (cmd_buf, "#%lx\n \n", fid);
  else
    // player is not paused, add to existing playlist
    len = sprintf (cmd_buf, "#%lx+\n \n", fid);

  hijack_serial_rx_insert (cmd_buf, len, 1);	// send command to player

  return 4;
}

static int seeburg_release (struct inode *inode, struct file *filep)
{
  sampler_state = 0;
  seeburg_sem = MUTEX_LOCKED;
  sampler_state = 0;

  return 0;
}
